/*

*/
import java.awt.Color;

public class MainCylinderSection
{
   public static void main(String[] args)
   {
      int k = 0; // number of triangles in the triangle fans at the top and bottom
      int n = 0; // number of triangle strips between the top and bottom

      // Check for two command line arguments.
      if (args.length >=2)
      {
         k = Integer.parseInt( args[0] );
         n = Integer.parseInt( args[1] );
      }
      else
      {
         System.err.printf("Usage: %s <k> <n>", MainCylinderSection.class.getName());
         System.exit(-1);;
      }

      // Create a Material object (brass).
      Material mtrl = new Material();
      mtrl.ambient[0] = 0.33f;
      mtrl.ambient[1] = 0.22f;
      mtrl.ambient[2] = 0.03f;
      mtrl.ambient[3] = 1.00f;
      mtrl.diffuse[0] = 0.78f;
      mtrl.diffuse[1] = 0.57f;
      mtrl.diffuse[2] = 0.11f;
      mtrl.diffuse[3] = 1.00f;
      mtrl.specular[0] = 0.99f;
      mtrl.specular[1] = 0.91f;
      mtrl.specular[2] = 0.81f;
      mtrl.specular[3] = 1.00f;
      mtrl.s = 27.8f;

      // Create a Light object for the scene.
      Light light = new Light();
      light.pos[0] = 0.0f;
      light.pos[1] = 0.0f;
      light.pos[2] = 10.0f;
      light.pos[3] = 1.0f;
      light.ambient[0] = 1.0f;
      light.ambient[1] = 1.0f;
      light.ambient[2] = 1.0f;
      light.ambient[3] = 1.0f;
      light.diffuse[0] = 1.0f;
      light.diffuse[1] = 1.0f;
      light.diffuse[2] = 1.0f;
      light.diffuse[3] = 1.0f;
      light.specular[0] = 1.0f;
      light.specular[1] = 1.0f;
      light.specular[2] = 1.0f;
      light.specular[3] = 1.0f;

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the light to the scene.
      scene.light = light;

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      //System.err.println( scene.toString(3) );

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);

      int numberOfFrames = 72;
      for (int i = 0; i <= numberOfFrames; i++)
      {
         Model m = new CylinderSectionModel(k, n, 1, 4,
                               2*Math.PI-(i*360.0/numberOfFrames)*(Math.PI/180), 2*Math.PI);
         for (Triangle t : m.triangleList)
            t.setMaterial( mtrl );
         m.doLighting = true;
         m.doBackFaceCulling = false;  // Important: try changing this.

         // Add the model to the Scene.
         scene.modelList.add(m);

         // Clear the framebuffer.
         fb.clearFB(java.awt.Color.darkGray);
         // Render our scene into the frame buffer.
         SceneRender.renderScene(scene, fb);
         // Save the resulting image in a file.
         fb.dumpFB2File( String.format("PPM_CylinderSection_Frame%03d.ppm", i) );

         // Remove the model from the scene.
         scene.modelList.remove(0);
      }

      for (int i = 0; i <= numberOfFrames; i++)
      {
         Model m = new CylinderSectionModel(k, n, 1, 4,
                               0, 2*Math.PI-(i*360.0/numberOfFrames)*(Math.PI/180));
         for (Triangle t : m.triangleList)
            t.setMaterial( mtrl );
         m.doLighting = true;
         m.doBackFaceCulling = false;  // Important: try changing this.
         m.modelRotate((360/numberOfFrames)*i, 1,0,0);

         // Add the model to the Scene.
         scene.modelList.add(m);

         // Clear the framebuffer.
         fb.clearFB(java.awt.Color.darkGray);
         // Render our scene into the frame buffer.
         SceneRender.renderScene(scene, fb);
         // Save the resulting image in a file.
         fb.dumpFB2File( String.format("PPM_CylinderSection_Frame%03d.ppm", numberOfFrames+1+i) );

         // Remove the model from the scene.
         scene.modelList.remove(0);
      }
   }
}