/*

*/

public class MainSphere
{
   public static void main(String[] args)
   {
      int k = 0; // number of triangles in the triangle fans at the top and bottom
      int n = 0; // number of triangle strips between the top and bottom

      // Check for two command line arguments.
      if (args.length >= 2)
      {
         k = Integer.parseInt( args[0] );
         n = Integer.parseInt( args[1] );
      }
      else
      {
         System.err.printf("Usage: %s <k> <n>", MainSphere.class.getName());
         System.exit(-1);;
      }

      // Create a Model object to hold the geometry.
      Model model = new Model();

      // Give the Model a model matrix.
      model.model2Identity();

       // Create the geometry for the Model.
      Vertex[][] v = new Vertex[n+1][k];

      // An array of vertices to be used to create triangles.
      for (int j = 0; j < k; j++)
      {
         double c = Math.cos(j*(2.0*Math.PI)/k);
         double s = Math.sin(j*(2.0*Math.PI)/k);
         for (int i = 0; i <= n; i++)
         {
            v[i][j] = new Vertex( Math.sin((i+1)*Math.PI/(n+2))*c,
                                  Math.cos((i+1)*Math.PI/(n+2)),
                                  Math.sin((i+1)*Math.PI/(n+2))*s);
         }
      }

      // Give the vertices some color.
      for (int i = 0; i <= n; i++)
      {
         for (int j = 0; j < k; j++)
         {
            v[i][j].setColor( Math.abs(v[i][j].x), Math.abs(v[i][j].y), Math.abs(v[i][j].z) );
         }
      }

      // Create the triangle fan at the top.
      Vertex topCenter = new Vertex(0.0, 1.0, 0.0);
      for (int j = 0; j < k-1; j++)
      {
         model.triangleList.add( new Triangle( topCenter, v[0][j+1], v[0][j] ) );
      }
      model.triangleList.add( new Triangle( topCenter, v[0][0], v[0][k-1] ) );

      // Create all the triangle strips between the top and bottom triangle fans.
      for (int i = 0; i < n; i++)
      {
         for (int j = 0; j < k-1; j++)
         {
            model.triangleList.add( new Triangle( v[i][j],     v[i][j+1], v[i+1][j+1] ) );
            model.triangleList.add( new Triangle( v[i+1][j+1], v[i+1][j], v[i][j]     ) );
         }
         model.triangleList.add( new Triangle( v[i][k-1], v[i][0],     v[i+1][0] ) );
         model.triangleList.add( new Triangle( v[i+1][0], v[i+1][k-1], v[i][k-1] ) );
      }

      // Create the triangle fan at the bottom.
      Vertex bottomCenter = new Vertex(0.0, -1.0, 0.0);
      for (int j = 0; j < k-1; j++)
      {
         model.triangleList.add( new Triangle( bottomCenter, v[n][j], v[n][j+1] ) );
      }
      model.triangleList.add( new Triangle( bottomCenter, v[n][k-1], v[n][0] ) );


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.modelList.add(model);

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      //System.err.println( scene.toString(3) );

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);
      // Render our scene into the frame buffer.
      SceneRender.renderScene(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_Sphere_Frame%02d.ppm", 0) );

      // Spin the model 360 degrees.
      for (int i = 1; i <= 36; i++)
      {
         // Rotate the model by 10 degrees.
         model.modelRotate(10, 0, 1, 0);
         // Render again.
         fb.clearFB(java.awt.Color.darkGray);
         SceneRender.renderScene(scene, fb);
         fb.dumpFB2File(String.format("PPM_Sphere_Frame%02d.ppm", i));
      }
   }
}