/*
                   y
                   |
                   |
                   | v5
                 3 +-----------------+ v6=(3,3,0)
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
        v4 +-----------------+ v3    |
           |       |         |       |                    y
           |       |         |       |                    |
           |       |         |       | v7                 |
           |    v8 +---------|-------+------> x           |
           |      /          |      /3                    |
           |    /            |    /                       +-----> x
           |  /              |  /                        /
           |/                |/                         /
         3 +-----------------+                         /
         / v1=(0,0,3)        v2=(3,0,3)              z
       /
     /
    z
*/

public class MainTexture
{
   public static void main(String[] args)
   {
      final int SIZE = 256;

      // Create a Model object to hold the geometry.
      Model model = new Model();

      // Give the Model a model matrix.
      model.model2Identity();

      // Add a texture to the Model.
      Texture texture1 = new Texture("resources/Yoda.ppm");
      model.addTexture( texture1 );

      // Add the geometry to the Model.
      Vertex v1 = new Vertex(0.0, 0.0, 0.0);
      Vertex v2 = new Vertex(0.0, 3.0, 0.0);
      Vertex v3 = new Vertex(3.0, 3.0, 0.0);
      Vertex v4 = new Vertex(3.0, 0.0, 0.0);

      v1.setCoords(0.0, 0.0);
      v2.setCoords(0.0, 1.0);
      v3.setCoords(1.0, 1.0);
      Triangle t1 = new Triangle(v1, v2, v3);
      t1.setTexture( texture1 );
      model.triangleList.add( t1 );  // front face

      v3.setCoords(1.0, 1.0);
      v4.setCoords(1.0, 0.0);
      v1.setCoords(0.0, 0.0);
      Triangle t2 = new Triangle(v3, v4, v1);
      t2.setTexture( texture1 );
      model.triangleList.add( t2 );   // front face


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.modelList.add(model);

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      //System.err.println( scene.toString(5) );

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);
      // Render our scene into the frame buffer.
      SceneRender.renderScene(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_Texture_Frame%02d.ppm", 0) );

      // Spin the model 360 degrees.
      for (int i = 1; i <= 36; i++)
      {
         // Rotate the model by 10 degrees.
         model.modelRotate(10, 0, 1, 0);
         // Render again.
         fb.clearFB(java.awt.Color.darkGray);
         SceneRender.renderScene(scene, fb);
         fb.dumpFB2File(String.format("PPM_Texture_Frame%02d.ppm", i));
      }
   }
}