/*
                   y
                   |
                   |
                   | v5
                   +-----------------+ v6=(3,3,0)
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
        v4 +-----------------+ v3    |
           |       |         |       |                    y
           |       |         |       |                    |
           |       |         |       | v7                 |
           |    v8 +---------|-------+------> x           |
           |      /          |      /                     |
           |    /            |    /                       +-----> x
           |  /              |  /                        /
           |/                |/                         /
           +-----------------+                         /
         / v1=(0,0,3)        v2=(3,0,3)              z
       /
     /
    z
*/

public class MainTextureCube
{
   public static void main(String[] args)
   {
      final int SIZE = 256;
      byte[] pascalImage = new byte[SIZE * SIZE * 3]; // a procedural texture

      // Create a Model object to hold the geometry.
      Model model = new Model();

      // Give the Model a model matrix.
      model.model2Identity();

      // Add three textures to the Model.
      // Create a "procedural" texture, pixel by pixel.
      for (int y = 0; y < SIZE; y++)
      {
         for (int x = 0; x < SIZE; x++)
         {
            byte c = (byte)((x|y)%255);  // Pascal's Triangle like image.
            int index = (y*SIZE + x)*3;
            pascalImage[index + 0] = c;
            pascalImage[index + 1] = c;
            pascalImage[index + 2] = c;
         }
      }
      Texture texture1 = new Texture("resources/Yoda.ppm");
      Texture texture2 = new Texture("resources/wood.ppm");
      Texture texture3 = new Texture(SIZE, SIZE, pascalImage);
      model.addTexture( texture1 );
      model.addTexture( texture2 );
      model.addTexture( texture3 );


      // Add the geometry to the Model.
      Vertex v1 = new Vertex(0.0, 0.0, 3.0);
      Vertex v2 = new Vertex(3.0, 0.0, 3.0);
      Vertex v3 = new Vertex(3.0, 3.0, 3.0);
      Vertex v4 = new Vertex(0.0, 3.0, 3.0);
      Vertex v5 = new Vertex(0.0, 3.0, 0.0);
      Vertex v6 = new Vertex(3.0, 3.0, 0.0);
      Vertex v7 = new Vertex(3.0, 0.0, 0.0);
      Vertex v8 = new Vertex(0.0, 0.0, 0.0);


      // Three textured sides of the cube.
      v1.setCoords(0.0, 0.0);
      v2.setCoords(1.0, 0.0);
      v3.setCoords(1.0, 1.0);
      v4.setCoords(0.0, 1.0);
      Triangle t1 = new Triangle(v1, v2, v3);
      t1.setTexture( texture1 );     // yoda face
      model.triangleList.add( t1 );  // front face

      Triangle t2 = new Triangle(v3, v4, v1);
      t2.setTexture( texture1 );
      model.triangleList.add( t2 );   // front face


      v3.setCoords(1.0, 0.0);
      v4.setCoords(0.0, 0.0);
      v5.setCoords(0.0, 1.0);
      v6.setCoords(1.0, 1.0);
      Triangle t3 = new Triangle(v4, v3, v6);
      t3.setTexture( texture2 );      //  wood face
      model.triangleList.add( t3 );   // top face

      Triangle t4 = new Triangle(v6, v5, v4);
      t4.setTexture( texture2 );
      model.triangleList.add( t4 );   // top face


      v2.setCoords(0.0, 0.0);
      v3.setCoords(0.0, 1.0);
      v6.setCoords(1.0, 1.0);
      v7.setCoords(1.0, 0.0);
      Triangle t5 = new Triangle(v2, v7, v6);
      t5.setTexture( texture3 );      // pascal face
      model.triangleList.add( t5 );   // right face

      Triangle t6 = new Triangle(v6, v3, v2);
      t6.setTexture( texture3 );
      model.triangleList.add( t6 );   // right face



      // Three non-textured sides of the cube.
      v1.setColor(0.0, 1.0, 0.0);
      v4.setColor(0.0, 0.0, 1.0);
      v5.setColor(0.0, 1.0, 0.0);
      v8.setColor(1.0, 0.0, 0.0);     // rainbow face
      Triangle t7 = new Triangle(v8, v1, v4);
      t7.setTexture( null );
      model.triangleList.add( t7 );   // left face

      Triangle t8 = new Triangle(v4, v5, v8);
      t8.setTexture( null );
      model.triangleList.add( t8 );   // left face


      v5.setColor(0.0, 1.0, 1.0);
      v6.setColor(0.0, 1.0, 1.0);
      v7.setColor(0.0, 1.0, 1.0);
      v8.setColor(0.0, 1.0, 1.0);     // yellow face
      Triangle t9 = new Triangle(v7, v8, v6);
      t9.setTexture( null );
      model.triangleList.add( t9 );   // back face

      Triangle t10 = new Triangle(v6, v5, v8);
      t10.setTexture( null );
      model.triangleList.add( t10 );   // back face


      v1.setColor(1.0, 1.0, 0.0);
      v2.setColor(1.0, 1.0, 0.0);
      v7.setColor(1.0, 1.0, 0.0);
      v8.setColor(1.0, 1.0, 0.0);     // magenta face
      Triangle t11 = new Triangle(v8, v1, v2);
      t11.setTexture( null );
      model.triangleList.add( t11 );   // bottom face

      Triangle t12 = new Triangle(v8, v2, v7);
      t12.setTexture( null );
      model.triangleList.add( t12 );   // bottom face


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.modelList.add(model);

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=5,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      //System.err.println( scene.toString(5) );

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);
      // Render our scene into the frame buffer.
      SceneRender.renderScene(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_TextureCube_Frame%02d.ppm", 0) );

      // Spin the model 360 degrees.
      for (int i = 1; i <= 36; i++)
      {
         // Rotate the model by 10 degrees.
         model.modelRotate(10, 0, 1, 0);
         // Render again.
         fb.clearFB(java.awt.Color.darkGray);
         SceneRender.renderScene(scene, fb);
         fb.dumpFB2File(String.format("PPM_TextureCube_Frame%02d.ppm", i));
      }
   }
}