/**

*/
import java.util.LinkedList;

public class Triangle
{
   public Vertex[] v = new Vertex[3]; // the vertices of this triangle
   public Material mtrl;              // data structure that describes material properties
   public Texture tex;                // a texture data structure

   public LinkedList<Fragment> fragments; // list of fragments (after rasterization stage)


   public Triangle()
   {
      v[0] = new Vertex();
      v[1] = new Vertex();
      v[2] = new Vertex();

      tex  = null;
      mtrl = null;
      fragments = null;  // points at fragments (after rasterization stage)
   }


   public Triangle(Vertex v0, Vertex v1, Vertex v2)
   {
      v[0] = new Vertex(v0);  // make copies of each Vertex!
      v[1] = new Vertex(v1);
      v[2] = new Vertex(v2);

      tex  = null;
      mtrl = null;
      fragments = null;  // points at fragments (after rasterization stage)
   }


   public Triangle(Triangle t) // a "copy constructor"
   {
      v[0] = new Vertex(t.v[0]);  // make copies of each Vertex
      v[1] = new Vertex(t.v[1]);
      v[2] = new Vertex(t.v[2]);

      tex  = t.tex;      // this is NOT a deep copy
      mtrl = t.mtrl;     // this is NOT a deep copy
      fragments = t.fragments; // this is NOT a deep copy
   }


   public void setTexture(Texture t)
   {
      this.tex = t;
   }


   public void setMaterial(Material m)
   {
      this.mtrl = m;
   }


   /**
      Give the triangle a uniform color.
   */
   public void setColor(java.awt.Color c)
   {
      v[0].setColor(c);
      v[1].setColor(c);
      v[2].setColor(c);
   }


   /**
      For debugging.
   */
   public String toString(int fields)
   {
      String result = "Triangle:\n";
      result += v[0].toString(fields);
      result += v[1].toString(fields);
      result += v[2].toString(fields);
      return result;
   }


   public String toString()
   {
      String result = "Triangle:\n";
      result += v[0].toString() + "\n";
      result += v[1].toString() + "\n";
      result += v[2].toString();
      return result;
   }
}