/**
   Convert triangles from view coordinates to clip coordinates
   relative to the standard clipping volume.
*/

public class P5_View2Clip
{
   /**
      Walk the list of Triangle objects.
      For each Vertex in each Triangle object,
      use the Scene's projection matrix to transform
      the Vertex object to clip coordinates.
   */
   public static void P5_View2Clip(Model model)
   {
      for (Triangle t : model.triangleList)
      {
         Matrix m = model.projMatrix;
         t.v[0] = m.times( t.v[0] );
         t.v[1] = m.times( t.v[1] );
         t.v[2] = m.times( t.v[2] );
         //debug(t);
      }
   }


   private static void debug(Triangle t)
   {
      // print the (x,y) part of each vertex in the triangle
      System.err.printf("x=% 6.2f, y=% 6.2f, x=% 6.2f, y=% 6.2f, x=% 6.2f, y=% 6.2f\n",
                         t.v[0].x, t.v[0].y, t.v[1].x, t.v[1].y, t.v[2].x, t.v[2].y);
      // "normalize" the triangle by doing the prespective division
      Vertex v0 = (t.v[0]).times( 1.0/(t.v[0].w) );
      Vertex v1 = (t.v[1]).times( 1.0/(t.v[1].w) );
      Vertex v2 = (t.v[2]).times( 1.0/(t.v[2].w) );
      // print the normalized (x,y) part of each vertex in the triangle
      System.err.printf("x=% 8.5f, y=% 8.5f, x=% 8.5f, y=% 8.5f, x=% 8.5f, y=% 8.5f\n",
                         v0.x, v0.y, v1.x, v1.y, v2.x, v2.y);
   }
}